=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-04-05 01:02:06 +0000
+++ bzrlib/commit.py	2006-04-05 18:05:15 +0000
@@ -309,7 +309,7 @@
                     or self.new_inv != self.basis_inv):
                 raise PointlessCommit()
 
-            if len(list(self.work_tree.conflict_lines()))>0:
+            if len(self.work_tree.conflicts())>0:
                 raise ConflictsInTree
 
             self.inv_sha1 = self.branch.repository.add_inventory(

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2006-04-05 04:54:00 +0000
+++ bzrlib/conflicts.py	2006-04-05 18:05:15 +0000
@@ -51,7 +51,7 @@
     def run(self):
         from bzrlib.workingtree import WorkingTree
         wt = WorkingTree.open_containing(u'.')[0]
-        for conflict in conflicts_to_strings(wt.conflict_lines()):
+        for conflict in conflicts_to_strings(wt.conflicts()):
             print conflict
 
 class cmd_resolve(bzrlib.commands.Command):
@@ -88,7 +88,7 @@
 def resolve(tree, paths=None, ignore_misses=False):
     tree.lock_write()
     try:
-        tree_conflicts = list(tree.conflict_lines())
+        tree_conflicts = list(tree.conflicts())
         if paths is None:
             new_conflicts = []
             selected_conflicts = tree_conflicts
@@ -96,7 +96,7 @@
             new_conflicts, selected_conflicts = \
                 select_conflicts(tree, paths, tree_conflicts, ignore_misses)
         try:
-            tree.set_conflict_lines(new_conflicts)
+            tree.set_conflicts(ConflictList(new_conflicts))
         except UnsupportedOperation:
             pass
         remove_conflict_files(tree, selected_conflicts)
@@ -188,13 +188,61 @@
         raise NotConflicted(filename)
 
 
+class ConflictList(object):
+    """List of conflicts
+    Can be instantiated from stanzas or from Conflict subclasses.
+    """
+
+    def __init__(self, conflicts=None):
+        object.__init__(self)
+        if conflicts is None:
+            self.__list = []
+        else:
+            self.__list = conflicts
+
+    def __len__(self):
+        return len(self.__list)
+
+    def __iter__(self):
+        return iter(self.__list)
+
+    def __getitem__(self, key):
+        return self.__list[key]
+
+    def append(self, conflict):
+        return self.__list.append(conflict)
+
+    def __eq__(self, other_list):
+        return list(self) == list(other_list)
+
+    def __ne__(self, other_list):
+        return not (self == other_list)
+
+    def __repr__(self):
+        return "ConflictList(%r)" % self.__list
+
+    @staticmethod
+    def from_stanzas(stanzas):
+        """Produce a new ConflictList from an iterable of stanzas"""
+        conflicts = ConflictList()
+        for stanza in stanzas:
+            conflicts.append(Conflict.factory(**stanza.as_dict()))
+        return conflicts
+
+    def to_stanzas(self):
+        """Generator of stanzas"""
+        for conflict in self:
+            yield conflict.as_stanza()
+            
+
 def conflicts_to_stanzas(conflicts):
-    for conflict in conflicts:
-        yield conflict.as_stanza()
+    for stanza in ConflictList(conflicts).to_stanzas():
+        yield stanza
+
 
 def stanzas_to_conflicts(stanzas):
-    for stanza in stanzas:
-        yield Conflict.factory(**stanza.as_dict())
+    for conflict in ConflictList.from_stanzas(stanzas):
+        yield conflict
 
 
 def conflicts_to_strings(conflicts):
@@ -215,14 +263,13 @@
             s.add('file_id', self.file_id)
         return s
 
+    def _cmp_list(self):
+        return [type(self), self.path, self.file_id]
+
     def __cmp__(self, other):
-        result = cmp(type(self), type(other))
-        if result != 0:
-            return result
-        result = cmp(self.path, other.path)
-        if result != 0:
-            return result
-        return cmp(self.file_id, other.file_id)
+        if getattr(other, "_cmp_list", None) is None:
+            return -1
+        return cmp(self._cmp_list(), other._cmp_list())
 
     def __eq__(self, other):
         return self.__cmp__(other) == 0
@@ -233,6 +280,11 @@
     def __str__(self):
         return self.format % self.__dict__
 
+    def __repr__(self):
+        rdict = dict(self.__dict__)
+        rdict['class'] = self.__class__.__name__
+        return self.rformat % rdict
+
     @staticmethod
     def factory(type, **kwargs):
         global ctype
@@ -240,8 +292,13 @@
 
 
 class PathConflict(Conflict):
+    """A conflict was encountered merging file paths"""
+
     typestring = 'path conflict'
+
     format = 'Path conflict: %(path)s / %(conflict_path)s'
+
+    rformat = '%(class)s(%(path)r, %(conflict_path)r, %(file_id)r)'
     def __init__(self, path, conflict_path=None, file_id=None):
         Conflict.__init__(self, path, file_id)
         self.conflict_path = conflict_path
@@ -254,20 +311,35 @@
 
 
 class ContentsConflict(PathConflict):
+    """The files are of different types, or not present"""
+
     typestring = 'contents conflict'
+
     format = 'Contents conflict in %(path)s'
 
 
 class TextConflict(PathConflict):
+    """The merge algorithm could not resolve all differences encountered."""
+
     typestring = 'text conflict'
+
     format = 'Text conflict in %(path)s'
 
 
 class HandledConflict(Conflict):
+    """A path problem that has been provisionally resolved.
+    This is intended to be a base class.
+    """
+
+    rformat = "%(class)s(%(action)r, %(path)r, %(file_id)r)"
+    
     def __init__(self, action, path, file_id=None):
         Conflict.__init__(self, path, file_id)
         self.action = action
 
+    def _cmp_list(self):
+        return Conflict._cmp_list(self) + [self.action]
+
     def as_stanza(self):
         s = Conflict.as_stanza(self)
         s.add('action', self.action)
@@ -275,12 +347,23 @@
 
 
 class HandledPathConflict(HandledConflict):
+    """A provisionally-resolved path problem involving two paths.
+    This is intended to be a base class.
+    """
+
+    rformat = "%(class)s(%(action)r, %(path)r, %(conflict_path)r,"\
+        " %(file_id)r, %(conflict_file_id)r)"
+
     def __init__(self, action, path, conflict_path, file_id=None,
                  conflict_file_id=None):
         HandledConflict.__init__(self, action, path, file_id)
         self.conflict_path = conflict_path 
         self.conflict_file_id = conflict_file_id
         
+    def _cmp_list(self):
+        return HandledConflict._cmp_list(self) + [self.conflict_path, 
+                                                  self.conflict_file_id]
+
     def as_stanza(self):
         s = HandledConflict.as_stanza(self)
         s.add('conflict_path', self.conflict_path)
@@ -291,27 +374,56 @@
 
 
 class DuplicateID(HandledPathConflict):
+    """Two files want the same file_id."""
+
     typestring = 'duplicate id'
+
     format = 'Conflict adding id to %(conflict_path)s.  %(action)s %(path)s.'
 
 
 class DuplicateEntry(HandledPathConflict):
+    """Two directory entries want to have the same name."""
+
     typestring = 'duplicate'
+
     format = 'Conflict adding file %(conflict_path)s.  %(action)s %(path)s.'
 
 
 class ParentLoop(HandledPathConflict):
+    """An attempt to create an infinitely-looping directory structure.
+    This is rare, but can be produced like so:
+
+    tree A:
+      mv foo/bar
+    tree B:
+      mv bar/foo
+    merge A and B
+    """
+
     typestring = 'parent loop'
+
     format = 'Conflict moving %(conflict_path)s into %(path)s.  %(action)s.'
 
 
 class UnversionedParent(HandledConflict):
+    """An attempt to version an file whose parent directory is not versioned.
+    Typically, the result of a merge where one tree unversioned the directory
+    and the other added a versioned file to it.
+    """
+
     typestring = 'unversioned parent'
+
     format = 'Conflict adding versioned files to %(path)s.  %(action)s.'
 
 
 class MissingParent(HandledConflict):
+    """An attempt to add files to a directory that is not present.
+    Typically, the result of a merge where one tree deleted the directory and
+    the other added a file to it.
+    """
+
     typestring = 'missing parent'
+
     format = 'Conflict adding files to %(path)s.  %(action)s.'
 
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-04-05 04:54:00 +0000
+++ bzrlib/merge.py	2006-04-05 18:05:15 +0000
@@ -22,7 +22,7 @@
 
 import bzrlib
 from bzrlib.branch import Branch
-from bzrlib.conflicts import conflicts_to_strings
+from bzrlib.conflicts import conflicts_to_strings, ConflictList
 from bzrlib.delta import compare_trees
 from bzrlib.errors import (BzrCommandError,
                            BzrError,
@@ -377,7 +377,7 @@
             results = self.tt.apply()
             self.write_modified(results)
             try:
-                working_tree.set_conflict_lines(self.cooked_conflicts)
+                working_tree.set_conflicts(ConflictList(self.cooked_conflicts))
             except UnsupportedOperation:
                 pass
         finally:

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2006-04-05 04:54:00 +0000
+++ bzrlib/status.py	2006-04-05 18:05:15 +0000
@@ -138,7 +138,7 @@
         if new_is_working_tree:
             list_paths('unknown', new.unknowns(), specific_files, to_file)
             conflict_title = False
-            for conflict in conflicts_to_strings(wt.conflict_lines()):
+            for conflict in conflicts_to_strings(wt.conflicts()):
                 if conflict_title is False:
                     print >> to_file, "conflicts:"
                     conflict_title = True

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2006-04-05 04:54:00 +0000
+++ bzrlib/tests/test_conflicts.py	2006-04-05 18:05:15 +0000
@@ -26,7 +26,7 @@
 # TODO: Test commit with some added, and added-but-missing files
 # RBC 20060124 is that not tested in test_commit.py ?
 
-example_conflicts = [ 
+example_conflicts = ConflictList([ 
     ContentsConflict('patha', 'ida'), 
     TextConflict('patha'),
     PathConflict('pathb', 'pathc', 'idb'),
@@ -36,7 +36,7 @@
     ParentLoop('Cancelled move', 'pathe', 'path2e', None, 'id2e'),
     UnversionedParent('Versioned directory', 'pathf', 'idf'),
     MissingParent('Not deleting', 'pathg', 'idg'),
-]
+])
 
 
 class TestConflicts(TestCaseWithTransport):
@@ -52,13 +52,13 @@
         file('hello.sploo.BASE', 'w').write('yellow world')
         file('hello.sploo.OTHER', 'w').write('yellow world2')
         self.assertEqual(len(list(tree.list_files())), 6)
-        conflicts = list(tree.conflict_lines())
+        conflicts = tree.conflicts()
         self.assertEqual(len(conflicts), 2)
         self.assert_('hello' in conflicts[0].path)
         self.assert_('hello.sploo' in conflicts[1].path)
         restore('hello')
         restore('hello.sploo')
-        self.assertEqual(len(list(tree.conflict_lines())), 0)
+        self.assertEqual(len(tree.conflicts()), 0)
         self.assertFileEqual('hello world2', 'hello')
         assert not os.path.lexists('hello.sploo')
         self.assertRaises(NotConflicted, restore, 'hello')

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-04-05 04:29:35 +0000
+++ bzrlib/tests/test_workingtree.py	2006-04-05 18:05:15 +0000
@@ -208,24 +208,24 @@
     def create_format2_tree(self, url):
         return BzrDir.create_standalone_workingtree(url)
 
-    def test_conflict_lines_format2(self):
+    def test_conflicts_format2(self):
         # test backwards compatability
         tree = self.create_format2_tree('.')
-        self.assertRaises(errors.UnsupportedOperation, tree.set_conflict_lines,
+        self.assertRaises(errors.UnsupportedOperation, tree.set_conflicts,
                           None)
         file('lala.BASE', 'wb').write('labase')
         expected = ContentsConflict('lala')
-        self.assertEqual(list(tree.conflict_lines()), [expected])
+        self.assertEqual(list(tree.conflicts()), [expected])
         file('lala', 'wb').write('la')
         tree.add('lala', 'lala-id')
         expected = ContentsConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflict_lines()), [expected])
+        self.assertEqual(list(tree.conflicts()), [expected])
         file('lala.THIS', 'wb').write('lathis')
         file('lala.OTHER', 'wb').write('laother')
         # When "text conflict"s happen, stem, THIS and OTHER are text
         expected = TextConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflict_lines()), [expected])
+        self.assertEqual(list(tree.conflicts()), [expected])
         os.unlink('lala.OTHER')
         os.mkdir('lala.OTHER')
         expected = ContentsConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflict_lines()), [expected])
+        self.assertEqual(list(tree.conflicts()), [expected])

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-03-10 18:13:13 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-04-05 18:05:15 +0000
@@ -505,22 +505,22 @@
         tree._control_files.put('merge-hashes', StringIO('asdfasdf'))
         self.assertRaises(errors.MergeModifiedFormatError, tree.merge_modified)
 
-    def test_conflict_lines(self):
+    def test_conflicts(self):
         from bzrlib.tests.test_conflicts import example_conflicts
         tree = self.make_branch_and_tree('master')
         try:
-            tree.set_conflict_lines(example_conflicts)
+            tree.set_conflicts(example_conflicts)
         except UnsupportedOperation:
-            raise TestSkipped('set_conflict_lines not supported')
+            raise TestSkipped('set_conflicts not supported')
             
         tree2 = WorkingTree.open('master')
-        self.assertEqual(list(tree2.conflict_lines()), example_conflicts)
+        self.assertEqual(tree2.conflicts(), example_conflicts)
         tree2._control_files.put('conflicts', StringIO(''))
         self.assertRaises(errors.ConflictFormatError, 
-                          tree2.conflict_lines)
+                          tree2.conflicts)
         tree2._control_files.put('conflicts', StringIO('a'))
         self.assertRaises(errors.ConflictFormatError, 
-                          tree2.conflict_lines)
+                          tree2.conflicts)
 
     def make_merge_conflicts(self):
         from bzrlib.merge import merge_inner 
@@ -542,27 +542,28 @@
 
     def test_merge_conflicts(self):
         tree = self.make_merge_conflicts()
-        self.assertEqual(len(list(tree.conflict_lines())), 1)
+        self.assertEqual(len(tree.conflicts()), 1)
 
     def test_clear_merge_conflicts(self):
+        from bzrlib.conflicts import ConflictList
         tree = self.make_merge_conflicts()
-        self.assertEqual(len(list(tree.conflict_lines())), 1)
+        self.assertEqual(len(tree.conflicts()), 1)
         try:
-            tree.set_conflict_lines([])
+            tree.set_conflicts(ConflictList())
         except UnsupportedOperation:
             raise TestSkipped
-        self.assertEqual(len(list(tree.conflict_lines())), 0)
+        self.assertEqual(tree.conflicts(), ConflictList())
 
     def test_revert_clear_conflicts(self):
         tree = self.make_merge_conflicts()
-        self.assertEqual(len(list(tree.conflict_lines())), 1)
+        self.assertEqual(len(tree.conflicts()), 1)
         tree.revert(["blo"])
-        self.assertEqual(len(list(tree.conflict_lines())), 1)
+        self.assertEqual(len(tree.conflicts()), 1)
         tree.revert(["bloo"])
-        self.assertEqual(len(list(tree.conflict_lines())), 0)
+        self.assertEqual(len(tree.conflicts()), 0)
 
     def test_revert_clear_conflicts2(self):
         tree = self.make_merge_conflicts()
-        self.assertEqual(len(list(tree.conflict_lines())), 1)
+        self.assertEqual(len(tree.conflicts()), 1)
         tree.revert([])
-        self.assertEqual(len(list(tree.conflict_lines())), 0)
+        self.assertEqual(len(tree.conflicts()), 0)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-04-05 04:54:00 +0000
+++ bzrlib/workingtree.py	2006-04-05 18:05:15 +0000
@@ -50,8 +50,7 @@
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.branch import (Branch,
                            quotefn)
-from bzrlib.conflicts import (stanzas_to_conflicts, conflicts_to_stanzas, Conflict,
-                              CONFLICT_SUFFIXES)
+from bzrlib.conflicts import Conflict, ConflictList, CONFLICT_SUFFIXES
 import bzrlib.bzrdir as bzrdir
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 import bzrlib.errors as errors
@@ -870,7 +869,7 @@
     @deprecated_method(zero_eight)
     def iter_conflicts(self):
         """List all files in the tree that have text or content conflicts.
-        DEPRECATED.  Use conflict_lines instead."""
+        DEPRECATED.  Use conflicts instead."""
         return self._iter_conflicts()
 
     def _iter_conflicts(self):
@@ -1287,11 +1286,12 @@
         self._set_inventory(inv)
         mutter('wrote working inventory')
 
-    def set_conflict_lines(self, arg):
-        raise UnsupportedOperation(self.set_conflict_lines, self)
+    def set_conflicts(self, arg):
+        raise UnsupportedOperation(self.set_conflicts, self)
 
     @needs_read_lock
-    def conflict_lines(self):
+    def conflicts(self):
+        conflicts = ConflictList()
         for conflicted in self._iter_conflicts():
             text = True
             try:
@@ -1316,8 +1316,10 @@
                         text = False
                         break
             ctype = {True: 'text conflict', False: 'contents conflict'}[text]
-            yield Conflict.factory(ctype, path=conflicted, 
-                                   file_id=self.path2id(conflicted))
+            conflicts.append(Conflict.factory(ctype, path=conflicted,
+                             file_id=self.path2id(conflicted)))
+        return conflicts
+
 
 class WorkingTree3(WorkingTree):
     """This is the Format 3 working tree.
@@ -1354,22 +1356,22 @@
             return True
 
     @needs_write_lock
-    def set_conflict_lines(self, lines):
-        self._put_rio('conflicts', conflicts_to_stanzas(lines), 
+    def set_conflicts(self, conflicts):
+        self._put_rio('conflicts', conflicts.to_stanzas(), 
                       CONFLICT_HEADER_1)
 
     @needs_read_lock
-    def conflict_lines(self):
+    def conflicts(self):
         try:
             confile = self._control_files.get('conflicts')
         except NoSuchFile:
-            return []
+            return ConflictList()
         try:
             if confile.next() != CONFLICT_HEADER_1 + '\n':
                 raise ConflictFormatError()
         except StopIteration:
             raise ConflictFormatError()
-        return stanzas_to_conflicts(RioReader(confile))
+        return ConflictList.from_stanzas(RioReader(confile))
 
 
 def get_conflicted_stem(path):

